/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_oops.c -- oops/oops.log colorizers for CCZE
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

#include "ccze-compat.h"

static void ccze_oops_setup (void);
static void ccze_oops_shutdown (void);
static int ccze_oops_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_oops;

static char *
ccze_oops_process (pcre2_match_data *offsets)
{
  char *date, *sp1, *id, *field, *sp2, *value, *etc;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&sp1, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&id, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&field, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&sp2, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&value, &l);
  pcre2_substring_get_bynumber (offsets, 9, (unsigned char **)&etc, &l);
  
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_addstr (CCZE_COLOR_DEFAULT, sp1);

  ccze_addstr (CCZE_COLOR_PIDB, "[");
  ccze_addstr (CCZE_COLOR_PROC, id);
  ccze_addstr (CCZE_COLOR_PIDB, "]");

  ccze_addstr (CCZE_COLOR_KEYWORD, "statistics()");
  ccze_addstr (CCZE_COLOR_DEFAULT, ":");
  ccze_space ();

  ccze_addstr (CCZE_COLOR_FIELD, field);
  ccze_addstr (CCZE_COLOR_DEFAULT, sp2);
  ccze_addstr (CCZE_COLOR_DEFAULT, ":");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_NUMBERS, value);
  ccze_addstr (CCZE_COLOR_DEFAULT, etc);

  ccze_newline ();
  
  pcre2_substring_free (date);
  pcre2_substring_free (sp1);
  pcre2_substring_free (id);
  pcre2_substring_free (field);
  pcre2_substring_free (sp2);
  pcre2_substring_free (value);
  pcre2_substring_free (etc);
  
  return NULL;
}

static void
ccze_oops_setup (void)
{
  int error;
  size_t errptr;

  reg_oops = pcre2_compile
    ("^((Mon|Tue|Wed|Thu|Fri|Sat|Sun) "
     "(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) "
     "\\d+ \\d+:\\d+:\\d+ \\d+)(\\s+)\\[([\\dxa-fA-F]+)\\]"
     "statistics\\(\\): ([\\S]+)(\\s*): (\\d+)(.*)",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_oops_shutdown (void)
{
  pcre2_code_free (reg_oops);
}

static int
ccze_oops_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_oops, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_oops_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (oops, FULL, "Coloriser for oops proxy logs.");
