/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_procmail.c -- procmail-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

static void ccze_procmail_setup (void);
static void ccze_procmail_shutdown (void);
static int ccze_procmail_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_procmail;

static char *
ccze_procmail_process (const char *str, pcre2_match_data *offsets)
{
  char *header = NULL, *value = NULL, *space1 = NULL;
  char *space2 = NULL, *extra = NULL;
  int handled = 0;
  size_t l;
  ccze_color_t col = CCZE_COLOR_UNKNOWN;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&space1, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&header, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&value, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&space2, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&extra, &l);
  
  if (header)
    {
      if (!strcasecmp ("from", header) || !strcasecmp (">from", header))
        {
          col = CCZE_COLOR_EMAIL;
          handled = 1;
        }
      if (!strcasecmp ("subject:", header))
        {
          col = CCZE_COLOR_SUBJECT;
          handled = 1;
        }
      if (!strcasecmp ("folder:", header))
        {
          col = CCZE_COLOR_DIR;
          handled = 1;
        }
    }

  if (!handled)
    {
      pcre2_substring_free (header);
      pcre2_substring_free (value);
      pcre2_substring_free (extra);
      pcre2_substring_free (space1);
      pcre2_substring_free (space2);
      return strdup (str);
    }

  ccze_addstr (CCZE_COLOR_DEFAULT, space1);
  ccze_addstr (CCZE_COLOR_DEFAULT, header);
  ccze_space ();

  ccze_addstr (col, value);
  if (col == CCZE_COLOR_EMAIL)
    col = CCZE_COLOR_DEFAULT;
  ccze_addstr (col, space2);

  if (!strcasecmp ("folder:", header))
    col = CCZE_COLOR_SIZE;
  else if (!strcasecmp ("from", header))
    col = CCZE_COLOR_DATE;

  ccze_addstr (col, extra);
  ccze_newline();
    
  pcre2_substring_free (extra);
  pcre2_substring_free (header);
  pcre2_substring_free (value);
  pcre2_substring_free (space1);
  pcre2_substring_free (space2);

  return NULL;
}

static void
ccze_procmail_setup (void)
{
  int error;
  size_t errptr;

  reg_procmail = pcre2_compile
    ("^(\\s*)(>?From|Subject:|Folder:)?\\s(\\S+)(\\s+)?(.*)",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_procmail_shutdown (void)
{
  pcre2_code_free (reg_procmail);
}

static int
ccze_procmail_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_procmail, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_procmail_process (str, offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (procmail, FULL, "Coloriser for procmail(1) logs.");
