/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_sulog.c -- su-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <stdlib.h>

static void ccze_sulog_setup (void);
static void ccze_sulog_shutdown (void);
static int ccze_sulog_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_sulog;

static char *
ccze_sulog_process (pcre2_match_data *offsets)
{
  char *date, *islogin, *tty, *fromuser, *touser;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&islogin, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&tty, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&fromuser, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&touser, &l);

  ccze_addstr (CCZE_COLOR_DEFAULT, "SU ");
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, islogin);
  ccze_space ();
  switch (tty[0])
    {
    case '?':
      ccze_addstr (CCZE_COLOR_UNKNOWN, tty);
      break;
    default:
      ccze_addstr (CCZE_COLOR_DIR, tty);
      break;
    }
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, fromuser);
  ccze_addstr (CCZE_COLOR_DEFAULT, "-");
  ccze_addstr (CCZE_COLOR_USER, touser);
  
  ccze_newline ();

  pcre2_substring_free (date);
  pcre2_substring_free (islogin);
  pcre2_substring_free (tty);
  pcre2_substring_free (fromuser);
  pcre2_substring_free (touser);

  return NULL;
}

static void
ccze_sulog_setup (void)
{
  int error;
  size_t errptr;

  reg_sulog = pcre2_compile ("^SU (\\d{2}\\/\\d{2} \\d{2}:\\d{2}) ([\\+\\-]) "
                             "(\\S+) ([^\\-]+)-(.*)$", PCRE2_ZERO_TERMINATED,
                             0, &error, &errptr, NULL);
}

static void
ccze_sulog_shutdown (void)
{
  pcre2_code_free (reg_sulog);
}

static int
ccze_sulog_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_sulog, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_sulog_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (sulog, FULL, "Coloriser for su(1) logs.");
