/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_syslog.c -- Syslog-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

#include "ccze-compat.h"

static void ccze_syslog_setup (void);
static void ccze_syslog_shutdown (void);
static int ccze_syslog_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_syslog;

static char *
ccze_syslog_process (pcre2_match_data *offsets)
{
  char *date = NULL, *host = NULL, *send = NULL, *process = NULL;
  char *msg = NULL, *pid = NULL, *tmp = NULL, *toret;
  int use_free = 0;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&host, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&send, &l);
  
  if ((strstr (send, "last message repeated") && strstr (send, "times")) ||
      (strstr (send, "-- MARK --")))
    {
      msg = strdup (send);
      use_free = 1;
    }
  else
    {
      pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&process, &l);
      pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&msg, &l);
    }
      
  if (process)
    {
      char *t;
      if ((t = strchr (process, '[')))
	{
	  char *t2 = strchr (t, ']');

	  pid = strndup (&t[1], (size_t)(t2 - t - 1));
	  tmp = strndup (process, (size_t)(t - process));
	  pcre2_substring_free (process);
	  process = strdup (tmp);
	}
    }

  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_HOST, host);
  ccze_space ();
  
  if (process)
    {
      ccze_addstr (CCZE_COLOR_PROC, process);
      if (pid)
	{
	  ccze_addstr (CCZE_COLOR_PIDB, "[");
	  ccze_addstr (CCZE_COLOR_PID, pid);
	  ccze_addstr (CCZE_COLOR_PIDB, "]");
	  ccze_addstr (CCZE_COLOR_PROC, ":");
	}
      ccze_space ();
      toret = strdup (msg);
    }
  else
    toret = strdup (send);

  pcre2_substring_free (date);
  pcre2_substring_free (host);
  pcre2_substring_free (send);
  /* free (process); */
  free (pid);
  free (tmp);

  if (use_free)
    free (msg);
  else
    pcre2_substring_free (msg);

  return toret;
}

static void
ccze_syslog_setup (void)
{
  int error;
  size_t errptr;

  reg_syslog = pcre2_compile ("^(\\S*\\s{1,2}\\d{1,2}\\s\\d\\d:\\d\\d:\\d\\d)"
                              "\\s(\\S+)\\s+((\\S+:?)\\s(.*))$",
                              PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_syslog_shutdown (void)
{
  pcre2_code_free (reg_syslog);
}

static int
ccze_syslog_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_syslog, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_syslog_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (syslog, FULL, "Generic syslog(8) log coloriser.");
