/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_xferlog -- Xferlog colorizer for CCZE.
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <stdlib.h>

static void ccze_xferlog_setup (void);
static void ccze_xferlog_shutdown (void);
static int ccze_xferlog_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_xferlog;

static char *
ccze_xferlog_log_process (pcre2_match_data *offsets)
{
  char *curtime, *transtime, *host, *fsize, *fname, *transtype;
  char *actionflag, *direction, *amode, *user, *service, *amethod;
  char *auid, *status;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&curtime, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&transtime, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&host, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&fsize, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&fname, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&transtype, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&actionflag, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&direction, &l);
  pcre2_substring_get_bynumber (offsets, 9, (unsigned char **)&amode, &l);
  pcre2_substring_get_bynumber (offsets, 10, (unsigned char **)&user, &l);
  pcre2_substring_get_bynumber (offsets, 11, (unsigned char **)&service, &l);
  pcre2_substring_get_bynumber (offsets, 12, (unsigned char **)&amethod, &l);
  pcre2_substring_get_bynumber (offsets, 13, (unsigned char **)&auid, &l);
  pcre2_substring_get_bynumber (offsets, 14, (unsigned char **)&status, &l);
  
  ccze_addstr (CCZE_COLOR_DATE, curtime);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_GETTIME, transtime);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_HOST, host);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_GETSIZE, fsize);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DIR, fname);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_PIDB, transtype);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, actionflag);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, direction);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, amode);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, user);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_SERVICE, service);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, amethod);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, auid);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, status);

  ccze_newline ();

  pcre2_substring_free (curtime);
  pcre2_substring_free (transtime);
  pcre2_substring_free (host);
  pcre2_substring_free (fsize);
  pcre2_substring_free (fname);
  pcre2_substring_free (transtype);
  pcre2_substring_free (actionflag);
  pcre2_substring_free (direction);
  pcre2_substring_free (amode);
  pcre2_substring_free (user);
  pcre2_substring_free (service);
  pcre2_substring_free (amethod);
  pcre2_substring_free (auid);
  pcre2_substring_free (status);

  return NULL;
}

static void
ccze_xferlog_setup (void)
{
  int error;
  size_t errptr;

  /* FIXME: Does not handle spaces in filenames! */
  reg_xferlog = pcre2_compile
    ("^(... ... +\\d{1,2} +\\d{1,2}:\\d{1,2}:\\d{1,2} \\d+) (\\d+) ([^ ]+) "
     "(\\d+) (\\S+) (a|b) (C|U|T|_) (o|i) (a|g|r) ([^ ]+) ([^ ]+) " 
     "(0|1) ([^ ]+) (c|i)", PCRE2_ZERO_TERMINATED, 0, &error,
     &errptr, NULL);
}

static void
ccze_xferlog_shutdown (void)
{
  pcre2_code_free (reg_xferlog);
}

static int
ccze_xferlog_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_xferlog, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_xferlog_log_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (xferlog, FULL, "Generic xferlog coloriser.");
